AlertWindow = class(Turbine.UI.Window)

function AlertWindow:Constructor()
    local width = Settings.AWidth
    local height = Settings.AHeight
    local x = Settings.AX
    local y = Settings.AY
    local cooldownTimer = 0
    local Alerts = {}
    local resort = false

    local MaxItems = math.floor(height / Settings.ABaseHeight)

    Turbine.UI.Window.Constructor(self);
    self:SetSize(width,height);
    self:SetPosition(x,y);
    self:SetZOrder(1);

    self:SetWantsKeyEvents(false)
    self:SetMouseVisible(false)
    self:SetVisible(true)
    --self:SetWantsUpdates(true)
    Updater:Subscribe(self)

    self.Message = function(throwaway, duration, message, icon, cdVisible)
        Alerts[#Alerts + 1] = Alert(self, duration, message, icon, cdVisible)
        resort = true
        --Alerts[5] = Alert(self, 10, "Test", PuddleIconSmall, true)
        ChatOutput(message)
        cooldownTimer = duration
      end

    self.Clear = function()
        --self.messageBox:SetText("")
        --cooldownTimer = 0
        --self.imageBox1:SetBackground("")
        --self.imageBox2:SetBackground("")
    end

    self.ReUpdateAlertPositions = function()

        local itemCount = Tablelength(Alerts)
        if itemCount == 0 then return end
        local durations = {}
        for key, value in pairs(Alerts) do
            durations[#durations + 1] = {value, value:GetDuration()}
        end
        table.sort(durations, Compare)
        local j = 1
        for i = MaxItems, 1, -1 do
            durations[j][1]:SetPosition(0, (MaxItems - 1) * (Settings.ABaseHeight) - (j * Settings.ABaseHeight))
            j = j + 1
            if j > itemCount then return end
        end
    end

    
    self.ExecuteUpdate = function(sender, delta)
        for key, value in pairs(Alerts) do
            if value ~= nil then
                value:UpdateText(delta)
                if value:GetDuration() <= 0 then
                    value:Destroy()
                    Alerts[key] = nil
                    resort = true
                end
            end
        end
        if resort then
            self.ReUpdateAlertPositions()
            resort = false
        end
    end

    local moveBox = MoveBox(self, width, height, "Alert Window")
    
    self.ToggleMove = function()
        moveBox:ToggleMove()
    end
end

Alert = class(Turbine.UI.Control)
function Alert:Constructor(parent, duration, message, icon, cdVisible)
    --ChatOutput("Printing:")
    --ChatOutput("Message: " .. message)
    --ChatOutput("Duration: " .. duration)
    --ChatOutput("Icon: " .. icon)
    --ChatOutput("CdVisible: " .. cdVisible)

    Turbine.UI.Control.Constructor(self)
    local timer = duration
    local showTimers = cdVisible
    local oldtimer = duration
    local colourEvent = 2
    local showLeft = Settings.AL
    local showRight = Settings.AR
    self:SetParent(parent)
    self:SetSize(Settings.AWidth, Settings.ABaseHeight)
    self:SetPosition(0, 0)
    self:SetMouseVisible(false)

    local messageBox = Turbine.UI.Label()
    messageBox:SetParent(self)
    messageBox:SetSize(Settings.AWidth - 2 * Settings.ABaseHeight, Settings.ABaseHeight)
    messageBox:SetPosition(Settings.ABaseHeight,0)
    messageBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    messageBox:SetFontStyle(Turbine.UI.FontStyle.Outline)
    messageBox:SetMouseVisible(false)
    messageBox:SetVisible(true)
    messageBox:SetMarkupEnabled(true)
    messageBox:SetOutlineColor(OutlineColor)
    messageBox:SetFont(Settings.AFont)
    messageBox:SetText(message)
    --self.messageBox:SetBackColor(Turbine.UI.Color(0,0,0))

    local imageBox1 = Turbine.UI.Label()
    imageBox1:SetParent(self)
    imageBox1:SetSize(Settings.ABaseHeight, Settings.ABaseHeight)
    imageBox1:SetPosition(0, 0)
    imageBox1:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    imageBox1:SetFont(Settings.AFont)
    imageBox1:SetFontStyle(Turbine.UI.FontStyle.Outline)
    imageBox1:SetOutlineColor(Turbine.UI.Color(0, 0, 0))
    imageBox1:SetForeColor(Turbine.UI.Color(1, 1, 1))
    imageBox1:SetMouseVisible(false)

    local imageBox2 = Turbine.UI.Label()
    imageBox2:SetParent(self)
    imageBox2:SetSize(Settings.ABaseHeight, Settings.ABaseHeight)
    imageBox2:SetPosition(Settings.AWidth - Settings.ABaseHeight, 0)
    imageBox2:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    imageBox2:SetFont(Settings.AFont)
    imageBox2:SetOutlineColor(Turbine.UI.Color(0, 0, 0))
    imageBox2:SetForeColor(Turbine.UI.Color(1, 1, 1))
    imageBox2:SetMouseVisible(false)

    if icon ~= nil then
        if Settings.AL then
            imageBox1:SetBackground(icon)
            imageBox1:SetVisible(true)
        end
        if Settings.AR then
            imageBox2:SetBackground(icon)
            imageBox2:SetVisible(true)
        end
      else
        imageBox1:SetBackground("")
        imageBox1:SetVisible(true)
        imageBox2:SetBackground("")
        imageBox2:SetVisible(true)
      end
    
      if cdVisible then
        if Settings.AL then
            imageBox1:SetText(duration)
        end
        if Settings.AR then
            imageBox2:SetText(duration)
        end
      else 
        imageBox1:SetText("")
        imageBox2:SetText("")
      end

    self.Destroy = function()
        self:SetVisible(false)
        self:SetParent(nil)
    end

    self.UpdateText = function(sender, delta)

        timer = timer - delta
        
        if oldtimer - timer > 0.05 then
            if showTimers and showLeft then
                if timer <= colourEvent then
                    imageBox1:SetText(Round(timer,1))
                    imageBox1:SetForeColor(Turbine.UI.Color(1, 0, 0))
                    colourEvent = -1
                else
                    imageBox1:SetText(Round(timer,0))
                end
                --self.imageBox1:SetText(math.floor(timer))
                if showRight then
                    imageBox2:SetText(math.floor(timer))
                end
            end
            oldtimer = timer
        end
    end

    self.GetDuration = function()
        return(timer)
    end
end